@@ -1,7 +1,7 @@
 ## Domain Registry Interface, .AT policy
 ## Contributed by Michael Braunoeder from NIC.AT <mib@nic.at>
 ##
-## Copyright (c) 2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -19,12 +19,16 @@
 package Net::DRI::DRD::AT;
 
 use strict;
+use warnings;
+
 use base qw/Net::DRI::DRD/;
 
-use Net::DRI::Exception;
 use DateTime::Duration;
+use Net::DRI::Data::Contact::AT;
 
-our $VERSION=do { my @r=(q$Revision: 1.5 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.7 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+
+__PACKAGE__->make_exception_for_unavailable_operations(qw/domain_transfer_accept domain_transfer_refuse domain_renew contact_transfer_stop contact_transfer_query contact_transfer_accept contact_transfer_refuse contact_check/);
 
 =pod
 
@@ -54,7 +58,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2006,2007,2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2006,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -82,28 +86,25 @@ sub periods  { return map { DateTime::Duration->new(years => $_) } (1); }
 sub name     { return 'NICAT'; }
 sub tlds     { return ('at'); }
 sub object_types { return ('domain','contact'); }
-
-sub transport_protocol_compatible
-{
- my ($self,$to,$po)=@_;
- my $pn=$po->name();
- my $tn=$to->name();
-
- return 1 if (($pn eq 'EPP') && ($tn eq 'socket_inet'));
- return 1 if (($pn eq 'Whois') && ($tn eq 'socket_inet'));
- return;
-}
+sub profile_types { return qw/epp whois/; }
 
 sub transport_protocol_default
 {
- my ($drd,$ndr,$type,$ta,$pa)=@_;
- $type='epp' if (!defined($type) || ref($type));
- return Net::DRI::DRD::_transport_protocol_default_epp('Net::DRI::Protocol::EPP::Extensions::AT',$ta,$pa) if ($type eq 'epp');
- return ('Net::DRI::Transport::Socket',[{%Net::DRI::DRD::PROTOCOL_DEFAULT_WHOIS,remote_host=>'whois.nic.at'}],'Net::DRI::Protocol::Whois',[]) if (lc($type) eq 'whois');
+ my ($self,$type)=@_;
+
+ return ('Net::DRI::Transport::Socket',{},'Net::DRI::Protocol::EPP::Extensions::AT',{})              if $type eq 'epp';
+ return ('Net::DRI::Transport::Socket',{remote_host=>'whois.nic.at'},'Net::DRI::Protocol::Whois',{}) if $type eq 'whois';
+ return;
+}
+
+sub set_factories
+{
+ my ($self,$po)=@_;
+ $po->factories('contact',sub { return Net::DRI::Data::Contact::AT->new(@_); });
 }
 
 ####################################################################################################
-
+## TODO : this should be converted to the new framework !
 sub verify_name_domain
 {
  my ($self,$ndr,$domain)=@_;
@@ -115,46 +116,15 @@ sub verify_name_domain
  my $r=$self->SUPER::check_name($domain,$count);
  return $r if ($r);
 
- return 10 unless $self->is_my_tld($domain);
+ return 10 unless $self->is_my_tld($ndr,$domain,0);
 
  return 0;
 }
 
-sub is_my_tld
-{
-   my ($self,$ndr,$domain)=@_;
-   ($domain)=($ndr) unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
-   my ($tld) = uc($self->tlds());
-
-   return 1 if (uc($domain) =~ /.*$tld$/);
-   return 0;
-}
-
-sub verify_duration_transfer
-{
- my ($self,$ndr,$duration,$domain,$op)=@_;
- ($duration,$domain,$op)=($ndr,$duration,$domain) unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
-
- return 0 unless ($op eq 'start'); ## we are not interested by other cases, they are always OK
- return 0;
-}
-
-sub domain_operation_needs_is_mine
-{
- my ($self,$ndr,$domain,$op)=@_;
- ($domain,$op)=($ndr,$domain) unless (defined($ndr) && $ndr && (ref($ndr) eq 'Net::DRI::Registry'));
-
- return unless defined($op);
-
- return 1 if ($op=~m/^(?:update|delete)$/);
- return 0 if ($op eq 'transfer');
- return;
-}
-
 sub domain_withdraw
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($domain);
+ $self->enforce_domain_name_constraints($ndr,$domain,'withdraw');
 
  $rd={} unless (defined($rd) && (ref($rd) eq 'HASH'));
  $rd->{transactionname} = 'withdraw';
@@ -166,7 +136,7 @@ sub domain_withdraw
 sub domain_transfer_execute
 {
  my ($self,$ndr,$domain,$rd)=@_;
- $self->err_invalid_domain_name($domain) if $self->verify_name_domain($domain);
+ $self->enforce_domain_name_constraints($ndr,$domain,'transfer_execute');
 
  $rd={} unless (defined($rd) && (ref($rd) eq 'HASH'));
  $rd->{transactionname} = 'transfer_execute';
@@ -189,13 +159,6 @@ sub message_delete
  return $rc;
 }
 
-sub message_waiting
-{
- my ($self,$ndr)=@_;
- my $c=$self->message_count($ndr);
- return (defined($c) && $c)? 1 : 0;
-}
-
 sub message_count
 {
  my ($self,$ndr)=@_;
@@ -205,16 +168,5 @@ sub message_count
  return (defined($count) && $count)? $count : 0;
 }
 
-## unsupported transactions
-sub domain_transfer_accept  { Net::DRI::Exception->die(0,'DRD',4,'No domain transfer approve available in .AT'); }
-sub domain_transfer_refuse  { Net::DRI::Exception->die(0,'DRD',4,'No domain transfer reject in .AT'); }
-sub domain_renew { Net::DRI::Exception->die(0,'DRD',4,'No domain renew in .AT'); }
-
-sub contact_transfer_stop   { Net::DRI::Exception->die(0,'DRD',4,'No contact transfer cancel available in .AT'); }
-sub contact_transfer_query  { Net::DRI::Exception->die(0,'DRD',4,'No contact transfer query available in .AT'); }
-sub contact_transfer_accept { Net::DRI::Exception->die(0,'DRD',4,'No contact transfer approve available in .AT'); }
-sub contact_transfer_refuse { Net::DRI::Exception->die(0,'DRD',4,'No contact transfer reject in .AT'); }
-sub contact_check { Net::DRI::Exception->die(0,'DRD',4,'No contact check in .AT'); }
-
 ####################################################################################################
 1;
