@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Logging operations for Net::DRI
 ##
-## Copyright (c) 2009-2010,2012 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -10,6 +10,9 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
+#
+# 
+#
 ####################################################################################################
 
 package Net::DRI::Logging;
@@ -21,6 +24,8 @@ use base qw/Net::DRI::BaseClass/;
 use Net::DRI::Util;
 use Net::DRI::Exception;
 
+our $VERSION=do { my @r=(q$Revision: 1.3 $=~/\d+/g); sprintf '%d'.('.%02d' x $#r), @r; };
+
 __PACKAGE__->make_exception_if_not_implemented(qw/name setup_channel output/);
 
 ## Taken from Log::Log4Perl
@@ -38,8 +43,6 @@ sub new
  if (! exists $self->{encoding}         || ! defined $self->{encoding} )         { $self->{encoding}        ='UTF-8'; }
  if (! exists $self->{format_header}    || ! defined $self->{format_header} )    { $self->{format_header}   ='%FULLTIME [%ULEVEL] <%TYPE>'; }
  if (! exists $self->{format_transport} || ! defined $self->{format_transport} ) { $self->{format_transport}='%TRID %UDIRECTION %MESSAGE'; }
- if (! exists $self->{format_protocol}  || ! defined $self->{format_protocol} )  { $self->{format_protocol} ='%TRID %UDIRECTION %MESSAGE'; }
- if (! exists $self->{sanitize_data}    || ! defined $self->{sanitize_data} )    { $self->{sanitize_data}   = {}; }
  bless $self,$c;
  $self->level($self->{level}); ## convert the level token to a numerical value
  return $self;
@@ -64,7 +67,8 @@ sub string_header
 {
  my ($self,$level,$type,$data)=@_;
  my $f=$self->{format_header};
- $f=~s/%FULLTIME/Net::DRI::Util::fulltime()/eg;
+ my $t=Net::DRI::Util::fulltime();
+ $f=~s/%FULLTIME/$t/g;
  $f=~s/%ULEVEL/uc($level)/eg;
  $f=~s/%TYPE/$type/g;
  return $f;
@@ -84,9 +88,9 @@ sub string_data
    $ct=$msg->hint();
   } else
   {
-   if (! $msg->can('as_string')) { Net::DRI::Exception::method_not_implemented('as_string',ref $msg); }
+   if (! $msg->can('as_string')) { Net::DRI::Exception::err_method_not_implemented('as_string in '.ref $msg); }
   }
-  $msg=$msg->as_string($self->{sanitize_data});
+  $msg=$msg->as_string();
  }
 
  ## If this is deemed to be too brittle, a type() method could be added to Protocol/Message and correctly set to "xml" by Message classes in RRI,EPP,OpenSRS/XCP,IRIS/{XCP,LWZ}
@@ -144,6 +148,10 @@ __END__
 
 Net::DRI::Logging - Logging Operations for Net::DRI
 
+=head1 VERSION
+
+This documentation refers to Net::DRI::Logging version 1.02
+
 =head1 SYNOPSIS
 
 This module is never used directly, only its subclasses are used. See the subclasses
@@ -188,12 +196,6 @@ It can be changed anytime later by using the level() method
 
 if needed, name of encoding to use to convert data stream ; default: UTF-8
 
-=item sanitize_data
-
-an optional ref hash to know which part of logged data should be replaced to ensure confidentiality;
-for now it works only for EPP, with the hash key "session_password" and a true value, then content
-of <pw> and <newPW> are replaced by a string of * in logging data
-
 =back
 
 =item name()
@@ -224,7 +226,7 @@ See the C<new()> method.
 
 =head1 DEPENDENCIES
 
-This module has to be used inside the Net::DRI framework and needs the following components:
+This modules has to be used inside the Net::DRI framework and needs the following components:
 
 =over
 
@@ -273,7 +275,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright (c) 2009-2010,2012 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+Copyright (c) 2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
