@@ -1,6 +1,6 @@
 ## Domain Registry Interface, .CZ EPP NSSET extension commands
 ##
-## Copyright (c) 2008 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
+## Copyright (c) 2008,2009 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
 ##                    All rights reserved.
 ##
 ## This file is part of Net::DRI
@@ -27,7 +27,7 @@ use Net::DRI::Data::ContactSet;
 
 use DateTime::Format::ISO8601;
 
-our $VERSION=do { my @r=(q$Revision: 1.2 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.3 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -57,7 +57,7 @@ Tonnerre Lombard, E<lt>tonnerre.lombard@sygroup.chE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
+Copyright (c) 2008,2009 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -254,7 +254,6 @@ sub info_parse
 
 	my $ns = Net::DRI::Data::Hosts->new();
 	my $pd = DateTime::Format::ISO8601->new();
-	my $cf = $po->factories()->{contact};
 	my $cs = Net::DRI::Data::ContactSet->new();
         my @s;
 	my $c = $infdata->getFirstChild();
@@ -283,7 +282,7 @@ sub info_parse
 		}
 		elsif ($name eq 'status')
 		{
-			push(@s, Net::DRI::Protocol::EPP::parse_status($c));
+			push(@s,$po->parse_status($c));
 		}
 		elsif ($name eq 'authInfo')
 		{
@@ -327,7 +326,7 @@ sub info_parse
 		}
 		elsif ($name =~ /^(registrant|billing|admin|tech)$/)
 		{
-			$cs->add($cf->()->srid($c->getFirstChild()->getData()),
+			$cs->add($po->create_local_object('contact')->srid($c->getFirstChild()->getData()),
 				$name);
 		}
 	} continue { $c = $c->getNextSibling(); }
