@@ -1,7 +1,7 @@
 ## Domain Registry Interface, EURid Sunrise EPP extension for Net::DRI
 ## (from registration_guidelines_v1_0F-appendix2-sunrise.pdf )
 ##
-## Copyright (c) 2005,2007,2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2005,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -29,7 +29,7 @@ use Net::DRI::Protocol::EPP::Core::Domain;
 use Net::DRI::Protocol::EPP::Extensions::EURid::Domain;
 use Net::DRI::DRD::EURid;
 
-our $VERSION=do { my @r=(q$Revision: 1.11 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.13 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -59,7 +59,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2005,2007,2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2005,2007,2008,2009 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -97,8 +97,7 @@ sub info
  Net::DRI::Exception::usererr_insufficient_parameters('Apply_info action needs a reference') unless defined($reference) && $reference;
  Net::DRI::Exception::usererr_invalid_parameters('reference must be a xml normalizedstring from 1 to 100 characters long') unless Net::DRI::Util::xml_is_normalizedstring($reference,1,100);
 
- my @ns=@{$mes->ns->{domain}};
- $mes->command(['apply-info','domain:apply-info',sprintf('xmlns:domain="%s" xsi:schemaLocation="%s %s"',$ns[0],$ns[0],$ns[1])]);
+ $mes->command(['apply-info','domain:apply-info',sprintf('xmlns:domain="%s" xsi:schemaLocation="%s %s"',$mes->nsattrs('domain'))]);
  $mes->command_body([['domain:reference',$reference]]);
 }
 
@@ -108,12 +107,10 @@ sub info_parse
  my $mes=$po->message();
  return unless $mes->is_success();
 
- my $infdata=$mes->get_content('appInfoData',$mes->ns('domain'));
+ my $infdata=$mes->get_response('domain','appInfoData');
  return unless $infdata;
 
  my $cs=Net::DRI::Data::ContactSet->new();
- my $cf=$po->factories()->{contact};
-
  my $pd=DateTime::Format::ISO8601->new();
  my $c=$infdata->firstChild();
  while ($c)
@@ -133,13 +130,13 @@ sub info_parse
    $rinfo->{domain}->{$oname}->{$1}=$pd->parse_datetime($c->firstChild->getData());
   } elsif ($name eq 'domain:registrant')
   {
-   $cs->set($cf->()->srid($c->firstChild->getData()),'registrant');
+   $cs->set($po->create_local_object('contact')->srid($c->firstChild->getData()),'registrant');
   } elsif ($name eq 'domain:contact')
   {
-   $cs->add($cf->()->srid($c->firstChild->getData()),$c->getAttribute('type'));
+   $cs->add($po->create_local_object('contact')->srid($c->firstChild->getData()),$c->getAttribute('type'));
   } elsif ($name eq 'domain:ns')
   {
-   $rinfo->{domain}->{$oname}->{ns}=Net::DRI::Protocol::EPP::Core::Domain::parse_ns($c);
+   $rinfo->{domain}->{$oname}->{ns}=Net::DRI::Protocol::EPP::Core::Domain::parse_ns($po,$c);
   } elsif ($name eq 'domain:adr')
   {
    $rinfo->{domain}->{$oname}->{adr}=Net::DRI::Util::xml_parse_boolean($c->firstChild->getData());
@@ -217,7 +214,7 @@ sub apply_parse
  $rinfo->{_internal}->{must_reconnect}=1; ## All apply commands (successful or not) close the connection
  return unless $mes->is_success();
 
- my $credata=$mes->get_content('appData',$mes->ns('domain'));
+ my $credata=$mes->get_response('domain','appData');
  return unless $credata;
 
  $rinfo->{domain}->{$oname}->{exist}=1;
