@@ -2,7 +2,6 @@
 ##
 ## Copyright (c) 2008 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
 ## Copyright (c) 2008 Thorsten Glaser for Sygroup GmbH
-## Copyright (c) 2010,2013 Patrick Mevzek <netdri@dotandco.com>
 ##                    All rights reserved.
 ##
 ## This file is part of Net::DRI
@@ -13,14 +12,19 @@
 ## (at your option) any later version.
 ##
 ## See the LICENSE file that comes with this distribution for more details.
+#
+# 
+#
 ####################################################################################################
 
 package Net::DRI::Protocol::EPP::Extensions::PL::Message;
 
 use strict;
-use warnings;
 
 use Net::DRI::Exception;
+use DateTime::Format::ISO8601;
+
+our $VERSION=do { my @r=(q$Revision: 1.1 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -53,7 +57,6 @@ Thorsten Glaser
 
 Copyright (c) 2008 Tonnerre Lombard <tonnerre.lombard@sygroup.ch>.
 Copyright (c) 2008 Thorsten Glaser for Sygroup GmbH
-Copyright (c) 2010,2013 Patrick Mevzek <netdri@dotandco.com>
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -85,7 +88,6 @@ sub poll
  Net::DRI::Exception::usererr_invalid_parameters('In EPP, you can not specify the message id you want to retrieve') if defined($msgid);
  my $mes=$epp->message();
  $mes->command([['poll',{op=>'req'}]]);
- return;
 }
 
 sub parse_poll
@@ -99,7 +101,7 @@ sub parse_poll
  my $action;
 
  return unless $mes->is_success();
- return if $mes->result_is('COMMAND_SUCCESSFUL_QUEUE_EMPTY');
+ return if ($mes->result_code() == 1300);	# no messages in queue
  return unless (defined($msgid) && $msgid);
 
  my $mesdata = $mes->node_resdata();
@@ -112,7 +114,7 @@ sub parse_poll
 
   if ($cmdname eq 'pollAuthInfo') {
    my $ra = $rinfo->{message}->{$msgid}->{extra_info};
-   push @{$ra}, $cnode->toString(); ### ???
+   push @{$ra}, $cnode;
    $action = 'pollAuthInfo';
 
    foreach my $cnode ($cnode->childNodes) {
@@ -184,7 +186,6 @@ sub parse_poll
  }
  $rinfo->{message}->{$msgid}->{object_type} = $otype;
  $rinfo->{$otype}->{$oname}->{message}=$mesdata;
- return;
 }
 
 ####################################################################################################
