@@ -1,6 +1,6 @@
 ## Domain Registry Interface, Gandi Web Services Message
 ##
-## Copyright (c) 2008 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
+## Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>. All rights reserved.
 ##
 ## This file is part of Net::DRI
 ##
@@ -24,7 +24,7 @@ use Net::DRI::Protocol::ResultStatus;
 use base qw(Class::Accessor::Chained::Fast Net::DRI::Protocol::Message);
 __PACKAGE__->mk_accessors(qw(version method params result errcode errmsg));
 
-our $VERSION=do { my @r=(q$Revision: 1.3 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
+our $VERSION=do { my @r=(q$Revision: 1.4 $=~/\d+/g); sprintf("%d".".%02d" x $#r, @r); };
 
 =pod
 
@@ -54,7 +54,7 @@ Patrick Mevzek, E<lt>netdri@dotandco.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2008 Patrick Mevzek <netdri@dotandco.com>.
+Copyright (c) 2008,2009 Patrick Mevzek <netdri@dotandco.com>.
 All rights reserved.
 
 This program is free software; you can redistribute it and/or modify
@@ -79,6 +79,18 @@ sub new
  return $self;
 }
 
+sub as_string
+{
+ my ($self)=@_;
+ my @p=@{$self->params()};
+ my @pr;
+ foreach my $i (0..$#p)
+ {
+  push @pr,sprintf 'PARAM%d=%s',$i+1,$p[$i];
+ }
+ return sprintf "METHOD=%s\n%s\n",$self->method(),join("\n",@pr);
+}
+
 sub add_session
 {
  my ($self,$sd)=@_;
